'''
This script takes in binarized images from full runs of the FVF experiment, and returns a text file with data 
about how the radius and velocity of the interface change through the runs. It is essentially my version of 
the pre-existing script 'mean_radius_FVF.py'.

It contains an option that uses an algorithm to find the centre of the initial bubble starting from an 
initial guess, by finding a local minimum (which should be the centre, given the initial guess is reasonable) 
in amplitude of a cosine fit of the radial vs. angular coordinates about the pixels it searches.

Written by: Matthew Gerry
October, 2018
'''
 
from os import listdir
import sys
sys.path.append('FVF_code') #add to sys.path whatever directory contains FVF-related codes
from PIL import Image
from numpy import pi, cos, sqrt, arctan, transpose, asmatrix, concatenate, array, arange, savetxt
from scipy.optimize import curve_fit
from matplotlib import use
use('Agg')
from matplotlib import pyplot as plt
from pylab import average, std, pi, sqrt, arctan
from kill_outliers import kill_outliers


# # Switches and inputs # #

find_centre = True # toggle the centre-finding feature of the script. This feature makes the program take significantly longer, but ensures that the radius data is as reliable as possible. A guess by eye might be sufficient
    
path_to_image_folder = 'FVF_data/18-09-26'  #  a string specifying the path to the folder that will contain the input images
                            # folder and all the new folders with the processed images.

image_folder_name = '/5psi_B_2'  # the name of the folder with the input images.

# Place estimates for the x and y coordinates of the centre. The optimzation function will search around this point.
x_centre_guess = int(input('Initial guess for x-coordinate of centre-point'))
y_centre_guess = int(input('Initial guess for y-coordinate of centre-point'))

# # Centre-finding and data acquisition # #

pi_by_2 = pi/2

file_list = listdir(path_to_image_folder+image_folder_name)  
file_list = sorted([name for name in file_list if (name.endswith('.tif') and not name.startswith('.'))]) 
# pick out only tifs and not hidden files.  
# Files are sorted by frame number as long as only one shot of the camera is in the directory.
# Below we pick out the first image, which should be of the initial bubble in the run.

img = Image.open(path_to_image_folder+image_folder_name+'/'+file_list[0])  # create PIL image object

img_greyscale = img.convert('L')   # cast the image into grey scale 0-255, just in case

img_matrix = transpose(asmatrix(img_greyscale).copy())  # copy the greyscale values into numpy matrix format
                                                    # the transpose makes the coordinates of images
                                                    # match those of matrices
img.close()
img_greyscale.close()


# Define model function for curve fit
def fitcos(phi,R_0,a,phi_0):
    return R_0 + a*cos(phi-phi_0)
    
# The searching part
if find_centre == True:
    search_array = arange(-1,2,1)
    
    while True: 
        fit_results = [[],[],[]] # x_centre, y_centre, and corresponding amplitude of oscillation
        fit_params = [] # we we will also save all the fit parameters for each guess
        for i in search_array:
            x_coord = x_centre_guess + i
            for j in search_array:
                y_coord = y_centre_guess + j # set the x,y coordinates to each point in the 3x3 square
                
                print('Checking pixel at ' + str(x_coord) + ', ' + str(y_coord) + '.')
                
                radius = []  # make empty arrays to hold the pixel radius data for the choice of centre point
                angle = []
                
                for x in range(0, img_matrix.shape[0]):
                    for y in range(0, img_matrix.shape[1]):
                
                        if img_matrix[x,y] == 0:  # this is a black pixel (white is 255)
                
                            x_pos = float(x-x_coord)
                            y_pos = float(y-y_coord)
                        
                            radius = radius + [sqrt(x_pos**2 + y_pos**2)]  # append
                
                            if radius != 0.0:   # avoid zero radii, just in case 
                
                                if x_pos != 0.0:  # avoid a divide by zero situation
                                    if x_pos > 0.0:
                                        angle = angle + [arctan(y_pos/x_pos)] # in radians
                                    else:
                                        if y_pos > 0.0:
                                            angle = angle + [arctan(y_pos/x_pos) + pi]
                                        else:
                                            angle = angle + [arctan(y_pos/x_pos) - pi]
                                else:
                                    if y_pos > 0.0:
                                        angle = angle + [pi_by_2]
                                    else:
                                        angle = angle + [-pi_by_2]
                
                            else:  # angle is undefined for zero radius: just make it zero
                                angle = 0.0   # this should never happen
                
                # end of the loop over 1 frame
                
                # uses an interquartile radius test to eliminate outliers due to stray black pixels
                radius, angle = kill_outliers((radius,angle))  
                
                # fit the data with this guess of centre point to a cosine function
                p_opt, p_cov = curve_fit(fitcos, angle, radius, (100.0, 10.0, 1.0))
                amplitude = abs(p_opt[1]) # the amplitude of oscillations about the average radius
        
                
                fit_results[0] = fit_results[0] + [x_coord]
                fit_results[1] = fit_results[1] + [y_coord]
                fit_results[2] = fit_results[2] + [amplitude]
                
                fit_params = fit_params + [p_opt] # save the fit parameters for plotting purposes
                
                print('Amplitude = ' + str(amplitude) + ' pixels')
        
        # Return the best of the range x-centre and y-centre guesses
        opt_index = fit_results[2].index(min(fit_results[2]))
        x_centre = fit_results[0][opt_index]
        y_centre = fit_results[1][opt_index]
        
        if x_centre == x_centre_guess and y_centre==y_centre_guess:
            break # stop searching if the optimal point of this search area is its interior point -- it is a local minimum of amplitude
        else:
            print('The optimum point of this search area was on the boundary. Update search area.')
            x_centre_guess = x_centre
            y_centre_guess = y_centre # update the search area to centre around the optimal point for the previous search area
            
        
    print('The centre of the intial bubble is closest to ' + str(x_centre) + ', ' + str(y_centre) + '.')

if find_centre == False: # if the find_centre option is not being used, use the initial guess as the centre pixel
    x_centre = x_centre_guess
    y_centre = y_centre_guess

# Now acquire all the radius data

fplot = []  # frame numbers for plotting
rplot = []  # radii 
stdplot = []  # std dev 
rmaxplot =[]  # max radius
rminplot =[]  # min radius

iframe = 0   # index to number the frames starting from 0

for image_file_name in file_list:

    print('start on image \ '+str(iframe)+ '  ' + image_file_name)

    img = Image.open(path_to_image_folder+image_folder_name+'/'+image_file_name)  # create PIL image object

    img_greyscale = img.convert('L')   # cast the image into grey scale 0-255, just in case

    img_matrix = transpose(asmatrix(img_greyscale).copy())  # copy the greyscale values into numpy matrix format
                                                        # the transpose makes the coordinates of images
                                                        # match those of matrices
    img.close()
    img_greyscale.close()

    pi_by_2 = pi/2.0

    radius = []  # make empty arrays to hold the pixel radius data
    angle = []

    for x in range(0, img_matrix.shape[0]):
        for y in range(0, img_matrix.shape[1]):

            if img_matrix[x,y] == 0:  # this is a black pixel (white is 255)

                x_pos = float(x-x_centre)
                y_pos = float(y-y_centre)

                radius = radius + [sqrt(x_pos**2 + y_pos**2)]  # append

                if radius != 0.0:   # avoid zero radii, just in case 
    
                    if x_pos != 0.0:  # avoid a divide by zero situation
                        if x_pos > 0.0:
                            angle = angle + [arctan(y_pos/x_pos)] # in radians
                        else:
                            if y_pos > 0.0:
                                angle = angle + [arctan(y_pos/x_pos) + pi]
                            else:
                                angle = angle + [arctan(y_pos/x_pos) - pi]
                    else:
                        if y_pos > 0.0:
                            angle = angle + [pi_by_2]
                        else:
                            angle = angle + [-pi_by_2]
    
                else:  # angle is undefined for zero radius: just make it zero
                    angle = 0.0   # this should never happen

    # end of the loop over 1 frame

    # uses an interquartile radius test to eliminate outliers due to stray black pixels
    radius, angle = kill_outliers((radius,angle))  

    fplot = fplot + [iframe]
    rplot = rplot + [average(radius)]
    stdplot = stdplot + [std(radius)]  # standard deviation is a measure of how noncircular the bubble is.

    rmaxplot = rmaxplot + [max(radius)]
    rminplot = rminplot + [min(radius)]

#   Use a quick plot to check for outliers due to wild stray pixels
#   these should have been eliminated by kill_outliers function

#	plot(angle,radius,'.')
#	show()		

    print('completed image ' + str(iframe) )

    iframe += 1     # go to the next frame

# save the data to a file for further analysis later
savetxt(path_to_image_folder+image_folder_name + '_radius_data.txt', (fplot,rplot,stdplot,rmaxplot,rminplot))

# plot and save the radius (min, max, avg, std) and velocity data throughout the run
plt.plot(fplot,rplot, label='avg radius')
plt.plot(fplot,stdplot, label='std dev')
plt.plot(fplot,rmaxplot, label=r'$R_{max}$') 
plt.plot(fplot,rminplot, label= r'$R_{min}$')
plt.xlabel('frame number')
plt.ylabel('radius (in pixels)')
plt.title('Radii ' + image_folder_name)
plt.legend(loc='upper left')

plt.savefig(path_to_image_folder+image_folder_name+'_radii.pdf')
