'''
Use a macro in ImageJ to create a folder containing skeletonized versions of all the binarized images from 
one run of the FVF experiment. Do not include in this folder any images in which the edge of the folder has been cut off

This program takes in that folder of images and outputs the number of fingers as a function of frame number. 
It counts the fingers in each image by effectively answering the question, "For a particle in motion along the
edge of the bubble, how many times does the radial coordinate of its position peak in value throughout its journey?"
--that is, it effectively plots, for each image, radial distance against an index keeping track of the order in
 which the black pixels were counted, ensuring a single-valued function of which local maxima can be identified.

Written by: Matthew Gerry
February 2019
'''

from os import listdir, mkdir
import sys
sys.path.append('FVF_code') # add to sys.path whatever directory contains FVF-related codes
from PIL import Image
from numpy import pi, sqrt, transpose, asmatrix, savetxt, arange, linspace, array, e, arctan
from scipy.signal import find_peaks, savgol_filter
from scipy.optimize import curve_fit
from matplotlib import use
#use('Agg') # require this command to make plots on the remote server
from matplotlib import pyplot as plt
import time
from kill_outliers import kill_outliers

plotflag = False # toggle live-plotting--this is more of a gimmick, and should not usually be used
unfold_frames = True # toggle the feature that produces an unfolded (phi vs. r) plot for each frame and marks peaks with an x. These can be used to make videos demonstrating finger formation throughout a run.

pi_by_2 = pi/2
        

path_to_image_folder = 'FVF_data/FVF_skeletonized_data'   #  a string specifying the path to the folder that will contain the input images
                            # folder and all the new folders with the processed images.

image_folder_name = '/2psi_G_needle4_2_empty_pipe_deci10_skel'  # the name of the folder with the input images.
    
x_centre = 940 # run fond_centre code beforehand to fix these parameters
y_centre = 838

frame_rate = 120.0
decimate = 1

file_list = listdir(path_to_image_folder+image_folder_name)  
file_list = sorted([name for name in file_list if (name.endswith('.tif') and not name.startswith('.'))]) 

fplot, n_count = [], [] # create empty lists to fill with data


if plotflag == True:
    xdata, ydata = [],[] # set up the axes for the live plot
    plt.show()
    axes = plt.gca()
    axes.set_xlim(0,len(file_list))
    axes.set_ylim(0,1)
    line, = axes.plot(xdata, ydata, 'b.')

if unfold_frames == True: # make a directory to save the unfolded frames in
    mkdir(path_to_image_folder+image_folder_name+'/'+image_folder_name+'_unfolded_peaks')
    peaks_r_master, peaks_phi_master = [], []
 
iframe = 0 # begin analyzing images from frame 0
for image_file_name in file_list:
    print('start counting image  '+str(iframe)+ '  ' + image_file_name)

    img = Image.open(path_to_image_folder+image_folder_name+'/'+image_file_name)  # create PIL image object

    img_greyscale = img.convert('L')   # cast the image into grey scale 0-255, just in case

    img_matrix = transpose(asmatrix(img_greyscale).copy())  # copy the greyscale values into numpy matrix format
                                                        # the transpose makes the coordinates of images
                                                        # match those of matrices
    img.close()
    img_greyscale.close()
    
    x = x_centre 
    y = y_centre
    
    while img_matrix[x,y]!=0:  # find a black pixel at which to begin measuring radial distance
        if x<img_matrix.shape[0]-1:
            x+=1 # walk right from the centre until hitting a black pixel
        else:
            x = x_centre # if no black pixel right of centre on the starting row, shift down one row
            y+=1
    
    covered = [] # record pixel loci for which the program has measured and recorded r
    t = 0 # start 'time' at zero, add one each time a new data point in r is recorded
    r_dist = [] # array of radial distances
    if unfold_frames == True:
        angle = [] # array of polar angles
    

    while True:
        if img_matrix[x,y]==0: # black
            if (x,y) not in covered:
                search_min = 0
                search_max = 1
                search_array = arange(search_min,search_max,1) # use these values to define a square around each pixel in which to check colour
                covered = covered + [(x,y)]
                updated = False
                while updated == False:
                    if search_min > -16:
                        search_min-=1
                        search_max+=1
                        search_array = arange(search_min,search_max,1) #increase the size of the search area
                        for i in search_array: #shift over by a pixel
                            x_coord = x+i
                            for j in search_array:
                                y_coord = y+j
                                if img_matrix[x_coord,y_coord]!=0 or (x_coord,y_coord) in covered: # don't count white or already-counted pixels
                                    continue
                                else:
                                    x_pos = float(x_coord-x_centre)
                                    y_pos = float(y_coord-y_centre)
                                    r = sqrt(x_pos**2 + y_pos**2) #radial distance
                                    r_dist = r_dist + [r]   
                                    
                                    if unfold_frames == True: # record the polar angle of each point as well as the radial distance
                                        if r != 0.0:   # avoid zero radii, just in case 
                            
                                            if x_pos != 0.0:  # avoid a divide by zero situation
                                                if x_pos > 0.0:
                                                    angle = angle + [arctan(y_pos/x_pos)] # in radians
                                                else:
                                                    if y_pos > 0.0:
                                                        angle = angle + [arctan(y_pos/x_pos) + pi]
                                                    else:
                                                        angle = angle + [arctan(y_pos/x_pos) - pi] #add/subtract pi as needed to correctly assign angle from arctan function
                                            else:
                                                if y_pos > 0.0:
                                                    angle = angle + [pi_by_2]
                                                else:
                                                    angle = angle + [-pi_by_2]
                            
                                        else:  # angle is undefined for zero radius: just make it zero
                                            angle = angle + [0.0]   # this should never happen
                                        
                                    t+=1 # advance time
                                    x = x_coord # update x and y coordinates with next adjacent black pixel
                                    y = y_coord
                                    updated = True
                                    #print(x,y,r)
                                    break # cut off for loop whenever a black pixel is found
                            if (x,y) not in covered:
                                break # cut off for loop whenever a black pixel is found
                    else:
                        break
            if (x,y) in covered: # stop upon reaching a pixel from which there are no new adjacent pixels to cover
                break
    
    # smooth out the radial distance data set so peaks are not double-counted due to noise
    r_dist_smooth1 = savgol_filter(r_dist, 21, 3)
    r_dist_smooth2 = savgol_filter(r_dist_smooth1, 21, 2)
    r_dist_smooth = savgol_filter(r_dist_smooth2, 21, 2)
    
    # Find peaks using nearest-neighbours approach in smoothed data
    n_fingers = len(find_peaks(r_dist_smooth, height=50)[0])
    print('Number of fingers: ' +str(n_fingers))
    fplot = fplot + [iframe]
    n_count = n_count + [n_fingers]
    
    if unfold_frames == True:
        for i in range(len(angle)):
            if angle[i]<0:
                angle[i] = angle[i]+2*pi # move the gap caused by removing the pipe away from the centre of the data set
        
        peaks_r, peaks_phi = [],[] # create arrays of just the angles and radial distance of the peaks to plot on top of the unfolded bubble
        for i in find_peaks(r_dist_smooth)[0]:
            peaks_r = peaks_r + [r_dist_smooth[i]]
            peaks_phi = peaks_phi + [angle[i]]
            
        peaks_r_master = peaks_r_master + peaks_r
        peaks_phi_master = peaks_phi_master + peaks_phi
    
        plt.plot(angle, r_dist_smooth,',', color='black') 
        plt.plot(peaks_phi_master, peaks_r_master, ',', color='blue') # plot unfolded bubble with peaks as they travel
        plt.plot(peaks_phi, peaks_r, 'x', color='red')
        plt.ylim(0, 700)
        plt.xlabel('Polar angle')
        plt.ylabel('Radial distance (pixels)')
        plt.title('Frame '+str(iframe)+', n = '+str(n_fingers))
        plt.savefig(path_to_image_folder+image_folder_name+'/'+image_folder_name+'_unfolded_peaks/'+image_folder_name+'_unfolded_peaks'+str("%04d" % (iframe,))+'.tif') #pad file names with 4-digit numbers for ease of use in ffmpeg
        plt.clf()
    
    if plotflag==True:
        xdata.append(iframe) #update the live plot
        ydata.append(n_fingers)
        axes.set_ylim(0,max(ydata)+2)
        line.set_xdata(xdata)
        line.set_ydata(ydata)
        plt.draw()
        plt.pause(1e-17)
        time.sleep(0.02)
    
    iframe+=1 # move on to the next frame parameter forward

#savetxt(path_to_image_folder+image_folder_name+'auto_finger_data.txt',(fplot,n_count))

t_array = array(fplot)*decimate/frame_rate
n_count_err = linspace(2.0,2.0,len(t_array))


# plot number of fingers against frame number throughout the run
plt.errorbar(t_array,n_count,yerr=n_count_err,fmt='.',label='Counted fingers')
plt.title('Counted Fingers (auto) vs. Time for 28psi, 1.512 mm Spacing')
plt.xlabel('Time (s)')
plt.ylabel('n')
plt.legend(loc='lower right')
plt.show()    
    
