'''
This code finds the centre of a bubble in the FVF experiment based on an initial guess by fitting r vs phi to a cosine 
function, and choosing r that minimizes the amplitude of oscillation.

It searches the 3x3 square of pixels centred at an initial guess, comparing amplitudes of the fit at each. 
If the point giving rise to the lowest amplitude is found to be on the boundary of this square, it re-centres 
the search area on this point and repeats the process. The code iterates over this process until it finds a pixel 
for which all adjacent pixels give a larger fit amplitude. It assumes that this local minimum is the global minimum, 
which is a fair assumption given that this function should only have one local minimum: the 'true' centre, and moves 
will be toward this point as long as the initial guess is within the edges of the bubble.

Written by: Matthew Gerry
October, 2018
'''

from os import listdir
import sys
sys.path.append('Documents/School/PHY479/FVF_code') #add to sys.path whatever directory contains FVF-related codes
from PIL import Image
from numpy import pi, cos, sqrt, arctan, transpose, asmatrix, concatenate, array, arange
from scipy.optimize import curve_fit
from matplotlib import pyplot as plt
from kill_outliers import kill_outliers

unfolded_plot == False # if True, the program will produce a r vs phi plot (unfolded) of the bubble's boundary in the given frame with the optimal centre point chosen

# Import initial bubble image and locate centre
path_to_image_folder = 'Desktop/FVF_photos'   #  a string specifying the path to the folder that will contain the input images
                            # folder and all the new folders with the processed images.

image_folder_name = '/FVF_photos_empty_pipe'  # the name of the folder with the input images.

# Place estimates for the x and y coordinates of the centre. The optimzation function will search around this point.
x_centre_guess = 947
y_centre_guess = 840

frame = 0 # frame number. 0 will usually give the initial frame of the run

pi_by_2 = pi/2
        
# Define model function for curve fit
def fitcos(phi,R_0,a,phi_0):
    return R_0 + a*cos(phi-phi_0)

file_list = listdir(path_to_image_folder+image_folder_name)  
file_list = sorted([name for name in file_list if (name.endswith('.tif') and not name.startswith('.'))]) 
# pick out only tifs and not hidden files.  
# Files are sorted by frame number as long as only one shot of the camera is in the directory.
# Below we pick out the first image, which should be of the initial bubble in the run.

img = Image.open(path_to_image_folder+image_folder_name+'/'+file_list[frame])  # create PIL image object

img_greyscale = img.convert('L')   # cast the image into grey scale 0-255, just in case

img_matrix = transpose(asmatrix(img_greyscale).copy())  # copy the greyscale values into numpy matrix format
                                                    # the transpose makes the coordinates of images
                                                    # match those of matrices
img.close()
img_greyscale.close()


# The searching part
search_array = arange(-1,2,1)

while True: 
    fit_results = [[],[],[]] # x_centre, y_centre, and corresponding amplitude of oscillation
    fit_params = [] # we we will also save all the fit parameters for each guess
    for i in search_array:
        x_coord = x_centre_guess + i
        for j in search_array:
            y_coord = y_centre_guess + j # set the x,y coordinates to each point in the 3x3 square
            
            print('Checking pixel at ' + str(x_coord) + ', ' + str(y_coord) + '.')
            
            radius = []  # make empty arrays to hold the pixel radius data for the choice of centre point
            angle = []
            
            for x in range(0, img_matrix.shape[0]):
                for y in range(0, img_matrix.shape[1]):
            
                    if img_matrix[x,y] == 0:  # this is a black pixel (white is 255)
            
                        x_pos = float(x-x_coord)
                        y_pos = float(y-y_coord)
                    
                        radius = radius + [sqrt(x_pos**2 + y_pos**2)]  # append
            
                        if radius != 0.0:   # avoid zero radii, just in case 
            
                            if x_pos != 0.0:  # avoid a divide by zero situation
                                if x_pos > 0.0:
                                    angle = angle + [arctan(y_pos/x_pos)] # in radians
                                else:
                                    if y_pos > 0.0:
                                        angle = angle + [arctan(y_pos/x_pos) + pi]
                                    else:
                                        angle = angle + [arctan(y_pos/x_pos) - pi]
                            else:
                                if y_pos > 0.0:
                                    angle = angle + [pi_by_2]
                                else:
                                    angle = angle + [-pi_by_2]
            
                        else:  # angle is undefined for zero radius: just make it zero
                            angle = 0.0   # this should never happen
            
            # end of the loop over 1 frame
            
            # uses an interquartile radius test to eliminate outliers due to stray black pixels
            radius, angle = kill_outliers((radius,angle))  
            
            # fit the data with this guess of centre point to a cosine function
            p_opt, p_cov = curve_fit(fitcos, angle, radius, (100.0, 10.0, 1.0))
            amplitude = abs(p_opt[1]) # the amplitude of oscillations about the average radius
    
            
            fit_results[0] = fit_results[0] + [x_coord]
            fit_results[1] = fit_results[1] + [y_coord]
            fit_results[2] = fit_results[2] + [amplitude]
            
            fit_params = fit_params + [p_opt] # save the fit parameters for plotting purposes
            
            print('Amplitude = ' + str(amplitude) + ' pixels')
    
    # Return the best of the range x-centre and y-centre guesses
    opt_index = fit_results[2].index(min(fit_results[2]))
    x_centre = fit_results[0][opt_index]
    y_centre = fit_results[1][opt_index]
    
    if x_centre == x_centre_guess and y_centre==y_centre_guess:
        break # stop searching if the optimal point of this search area is its interior point -- it is a local minimum of amplitude
    else:
        print('The optimum point of this search area was on the boundary. Update search area.')
        x_centre_guess = x_centre
        y_centre_guess = y_centre # update the search area to centre around the optimal point for the previous search area
        
    
print('The centre of the intial bubble is closest to ' + str(x_centre) + ', ' + str(y_centre) + '.')

# plot the radius vs. angle with the optimal centre point chosen
if unfolded_plot == True:
    radius = []  # make empty arrays to hold the pixel radius data for the choice of centre
    angle = []
    
    for x in range(0, img_matrix.shape[0]):
    for y in range(0, img_matrix.shape[1]):
    
        if img_matrix[x,y] == 0:  # this is a black pixel (white is 255)
    
            x_pos = float(x-x_centre)
            y_pos = float(y-y_centre)
        
            radius = radius + [sqrt(x_pos**2 + y_pos**2)]  # append
    
            if radius != 0.0:   # avoid zero radii, just in case 
    
                if x_pos != 0.0:  # avoid a divide by zero situation
                    if x_pos > 0.0:
                        angle = angle + [arctan(y_pos/x_pos)] # in radians
                    else:
                        if y_pos > 0.0:
                            angle = angle + [arctan(y_pos/x_pos) + pi]
                        else:
                            angle = angle + [arctan(y_pos/x_pos) - pi]
                else:
                    if y_pos > 0.0:
                        angle = angle + [pi_by_2]
                    else:
                        angle = angle + [-pi_by_2]
    
            else:  # angle is undefined for zero radius: just make it zero
                angle = 0.0   # this should never happen
    
    # end of the loop over 1 frame
    
    # uses an interquartile radius test to eliminate outliers due to stray black pixels
    radius, angle = kill_outliers((radius,angle))
    
    # set parameters based on the optimal fit
    phi = arange(-pi, pi, 0.1)
    R_initial = fit_params[opt_index][0]
    A = fit_params[opt_index][1]
    phase = fit_params[opt_index][2]
    
    plt.plot(angle, radius, '.')
    plt.plot(phi, fitcos(phi,R_initial,A,phase))
    plt.title('R vs. phi (data and fit) for optimized centre point')
    #plt.show()