'''
This script was written to assess how the predicted values for the number of fingers change when the stipulated end of the linear regime is changed.

Written by: Matthew Gerry
December, 2018

'''

import sys
sys.path.append('FVF_code')
from numpy import loadtxt, polyfit, polyder, polyval, arange, sqrt, argmax, concatenate, round, median, savetxt
from matplotlib import pyplot as plt
from sigma_bv import *
from viscosity_vs_T import *

##

# Choose a specific run of the experiment
path_to_txt_name = 'Desktop/frame_stop_test/5psi_B_5'   #  a string specifying the path to the txt file of the mean radius data.
input_txt_name = '/5psi_B_5_empty_pipe_deci3_radius_data.txt'  # the name of the txt file with the mean radius data.
# retrieve mean radius data from the txt file that was created by mean_radius_FVF.py
(frame_num, rad_pixels, stdevn, rmax, rmin) = loadtxt(path_to_txt_name + input_txt_name)

decimate = int(input('Decimate: '))  # this has to match the value used in process_FVF_images.py 
frame_rate = 120.0  # the number of frames per second the camera was originally set to
sec_per_frame = decimate*(1.0/frame_rate)  #  Note how decimation increases the time between frames
mm_per_pixel = 20.0/70.0  # Count xx mm for yy pixels from your calibration image showing the plastic ruler            
meter_per_pixel = mm_per_pixel/1000.0  # better units

n_count = input('How many fingers were counted? ') #counted finger value
n_count_err = input('Uncertainty in counted fingers value: ') #error in counted finger value
n_int = []

b = 5.2e-4 #plate spacing in m

temperature = 22.0  # the temperature of the oil in degrees C
rho = 8.6e2 # the density of the oil in kg/m^3

gamma = 0.033       					# the surface tension in N/m.  See Table 1.
Ca_ratio = mu_vs_T(temperature)/gamma	# = mu/gamma in (s/m).  The conversion factor of velocity in (m/s) to capillary number Ca. 
Re_ratio = b*rho/mu_vs_T(temperature)

frame_start = 0
frame_stop_range = 25 #this sets the number of different fram_stop values to test
frame_stop_low = int(input('The earliest possible end to the linear regime: ')) #this is the first value of frame_stop the code will try
frame_stop = []

for i in range(frame_stop_range):
    frame_stop = frame_stop + [frame_stop_low + i] # cycle through different choices for the end of the linear regime
    # Truncate the data to just the linear regime and do units conversion from pixels/frame to m/s
    time = sec_per_frame * frame_num[ frame_start:frame_stop[i]+1 ]  		# array of times at which the radius data were collected, now in seconds
    radii = meter_per_pixel * rad_pixels[ frame_start:frame_stop[i]+1 ]	# array of radii, now in meters
    
    # # # fit radius and velocity data to polynomials, derive Ca data # # #
    
    order_of_polyfit = 4  # The order of the polynomial fit. Make this big enough to make a tight fit, but not more than that.
    # carry out the polynomial fit. This is just a phenomenological fit to allow us to differentiate the data nicely
    R_fit_coef = polyfit(time, radii, order_of_polyfit)  
    
    # fit the coefficients for the derivative of the polynomial fit --> a polynomial for v = dR/dt
    v_fit_coef = polyder(R_fit_coef)
    
    initial_radius = polyval(R_fit_coef, time)[0] #value should be similar to radii[0]
    initial_velocity = polyval(v_fit_coef, time)[1]
    
    R = polyval(R_fit_coef, time)
    v = polyval(v_fit_coef, time)
    
    Ca = Ca_ratio*v
    Ca_med = median(Ca)
    Ca_err = (max(Ca)-min(Ca))/2
    Ca_init = Ca[0]
    
    bR = b/R # dimensionless radius parameter
    
    # # # predictions for dominant modes # # #
    modes = arange(2,151)  # an array of integers between 2 and 150 inclusive
    
    if i == 0:
        #  calculate the value of n_max according to Eqn. 63, using the first values (smallest R)
        n_init = modes[argmax(sigma_bv(modes,Ca[0],bR[0]))] # dominant mode at the start (Maxworthy prediction)
    
    #  Now carry out the numerical integration of the the growth rate * dt for each of the modes
    
    int_sigma_dt = zeros(len(modes))  # array to hold the relative growth of the modes
    
    for i in range(len(modes)):  	# step through the mode numbers
        for j in range(len(time)):		# step through the time for each mode
    
            v = polyval(v_fit_coef, time[j]) # use the fit to find the velocity
    
            int_sigma_dt[i] = int_sigma_dt[i] + ( v/b ) * sigma_bv(modes[i],Ca[j],bR[j],posdef=True) * sec_per_frame  # add up sigma * dt
    
    growth = exp(int_sigma_dt)
    
    # locate the mode number which is has grown the most.  Others may be very close.
    
    max_growth = max(growth)
    n_max_growth = modes[argmax(growth)]
    n_int = n_int + [n_max_growth]

# Plot the prediction for a variety of different choices of frame_stop
plt.plot(frame_stop, n_int, '.')
plt.title('Predicted n vs frame_stop Choice, ' + input_txt_name + 'n_count = ' + str(n_count) +', n_init = ' + str(n_init))
plt.xlabel('frame_stop')
plt.ylabel('Predicted n')
plt.show()


