'''
This code takes data from a variety of runs together and produces a variety of different plots. 
Use in an IDE that allows separate sections of the code to be run separately. Load the data acquired using 
FVF_getradius.py in one file at a time. Then produce plots using the estimates for n generated by linear stability theory. 

Written by: Matthew Gerry
2018-19
'''

import sys
sys.path.append('FVF_code') #add to sys.path wherever FVF-related scripts are saved
from numpy import loadtxt, polyfit, polyder, polyval, arange, sqrt, argmax, concatenate, round, median, savetxt, std, average, append, array, linspace, logspace
from matplotlib import pyplot as plt
from sigma_bv import *
from viscosity_vs_T import *
from itertools import chain
from scipy.interpolate import griddata, interp2d

temperature = 22.0  # the temperature of the oil in degrees C
rho = 8.6e2 # the density of the oil in kg/m^3
gamma = 0.033       					# the surface tension in N/m.  See Table 1.

# spacer dimensions, in m
spacer_dim = {'A': 7.91e-4, 'B': 1.051e-3, 'C': 1.303e-3, 'D': 1.512e-3, 'E': 2.334e-3, 'F': 3.206e-3, 'G': 5.2e-4, 'H': 3.2e-4}

## Initialize lists for plotting, set parameters
runs_name = input('General name for the set of runs being plotted: ') #to name the text file holding the data

Ca_med = [] #median of Ca values for a given run, error bars will cover entire range
Ca_init = []
Ca_err = []
Re_av = []
Re_std = []
n_count = [] #number of fingers 
n_count_err = []
n_init = [] #predicted dominant mode from initial parameters
n_int = [] #predicted dominant mode from integration
Rb_init = []


## Add data from each run

path_to_txt_name = 'Desktop/G_2018'   #  a string specifying the path to the txt file of the mean radius data.
input_txt_name = '/3psi_G_needle6_IB3_14_empty_pipe_deci4_radius_data.txt'  # the name of the txt file with the mean radius data.

spacers = str('G') # label of spacers used
b = spacer_dim[spacers] # the plate spacing in meters
Ca_ratio = mu_vs_T(temperature)/gamma	# = mu/gamma in (s/m).  The conversion factor of velocity in (m/s) to capillary number Ca. 
Re_ratio = b*rho/mu_vs_T(temperature)

decimate = 4
#decimate = int(input('Decimate: '))  # this has to match the value used in process_FVF_images.py 
frame_rate = 120.0  # the number of frames per second the camera was originally set to
sec_per_frame = decimate*(1.0/frame_rate)  #  Note how decimation increases the time between frames
mm_per_pixel = 20.0/70.0  # Count xx mm for yy pixels from your calibration image showing the plastic ruler            
meter_per_pixel = mm_per_pixel/1000.0  # better units

# retrieve mean radius data from the txt file that was created by mean_radius_FVF.py
(frame_num, rad_pixels, stdevn, rmax, rmin) = loadtxt(path_to_txt_name + input_txt_name)

frame_start = 0     # this will be zero unless you started from a static circular bubble
frame_stop = int(input('At what frame does the linear regime end? '))     # inclusive  (add +1 below), the frame at which the linear regime ends in the given run

# Truncate the data to just the linear regime and do units conversion from pixels/frame to m/s
time = sec_per_frame * frame_num[ frame_start:frame_stop+1 ]  		# array of times at which the radius data were collected, now in seconds
radii = meter_per_pixel * rad_pixels[ frame_start:frame_stop+1 ]	# array of radii, now in meters

# # # acquire information about counted fingers # # #

counted = int(input('How many fingers were counted: '))
count_sigma = int(input('Uncertainty in counted number: '))
n_count = n_count + [counted]
n_count_err = n_count_err + [count_sigma]

# # # fit radius and velocity data to polynomials, derive Ca data # # #

order_of_polyfit = 4  # The order of the polynomial fit. Make this big enough to make a tight fit, but not more than that.
# carry out the polynomial fit. This is just a phenomenological fit to allow us to differentiate the data nicely
R_fit_coef = polyfit(time, radii, order_of_polyfit)  

# fit the coefficients for the derivative of the polynomial fit --> a polynomial for v = dR/dt
v_fit_coef = polyder(R_fit_coef)

initial_radius = polyval(R_fit_coef, time)[0] #value should be similar to radii[0]
initial_velocity = polyval(v_fit_coef, time)[1]

R = polyval(R_fit_coef, time)
v = polyval(v_fit_coef, time)

Ca = Ca_ratio*v
Ca_med = Ca_med + [median(Ca)]
Ca_err = Ca_err + [(max(Ca)-min(Ca))/2]
Ca_init = Ca_init + [Ca[0]]

Re = Re_ratio*v
Re_av = Re_av + [average(Re)]
Re_std = Re_std + [std(Re)] # USING STDEV AS ERROR FOR NOW

bR = b/R # dimensionless radius parameter
Rb_init = Rb_init + [initial_radius/b]

# # # predictions for dominant modes # # #
modes = arange(2,201)  # an array of integers between 2 and 150 inclusive

#  calculate the value of n_max according to Eqn. 63, using the first values (smallest R)
n_max = modes[argmax(sigma_bv(modes,Ca[0],bR[0]))] # dominant mode at the start (Maxworthy prediction)
n_init = n_init + [n_max]

#  Now carry out the numerical integration of the the growth rate * dt for each of the modes

int_sigma_dt = zeros(len(modes))  # array to hold the relative growth of the modes

for i in range(len(modes)):  	# step through the mode numbers
    for j in range(len(time)):		# step through the time for each mode

        v = polyval(v_fit_coef, time[j]) # use the fit to find the velocity

        int_sigma_dt[i] = int_sigma_dt[i] + ( v/b ) * sigma_bv(modes[i],Ca[j],bR[j],posdef=True) * sec_per_frame  # add up sigma * dt

growth = exp(int_sigma_dt)

# locate the mode number which is has grown the most.  Others may be very close.

max_growth = max(growth)
n_max_growth = modes[argmax(growth)]
n_int = n_int + [n_max_growth]


## Save finger counting data
savetxt('Documents/School/PHY479/FVF_finger_data/' + runs_name + '_finger_count_data.txt', (Ca_med, Ca_init, Ca_err, Re_av, Re_std, n_count, n_count_err, n_init, n_int, Rb_init), header='Ca_med, Ca_init, Ca_err, Re_av, Re_std, n_count, n_count_err, n_init, n_int, Rb_init')

## Plot predicted and counted fingers on linear plot
# n vs. Ca ranges on lin-lin plot

plt.errorbar(Ca_med, n_count, fmt='.', xerr=Ca_err, yerr=n_count_err, color='blue',label='Counted fingers')
plt.plot(Ca_init, n_init, 'o', color='green',label='Prediction (initial)')
plt.errorbar(Ca_med, n_int, xerr=Ca_err, fmt = '.', color='red',label='Prediction (integration)')
plt.title('Finger Count and Predictions with ' + spacers + ' Spacers--Linear')
plt.xlabel('Ca')
plt.ylabel('n')
plt.legend(loc = 'lower right')
plt.show()

## Plot predicted and counted fingers on semilog plot
# n vs. Ca ranges on semilog plot

plt.errorbar(Ca_med, n_count, xerr=Ca_err, yerr=n_count_err, fmt = ' ', color='blue',label='Counted fingers')
plt.plot(Ca_init, n_init, 'o', color='green',label='Prediction (initial)')
plt.errorbar(Ca_med, n_int, xerr=Ca_err, fmt = ' ', color='red',label='Prediction (integration)')
plt.title('Finger Count and Predictions with ' + spacers + ' Spacers, Initial R/b: ~30-35--Semilog')
plt.xscale('log')
plt.xlabel('Ca')
plt.ylabel('n')
plt.legend(loc = 'lower right')
plt.show()

## Plot predictions against counts
# diagonal plots

plt.errorbar(n_count, n_int, xerr=n_count_err, fmt ='.', color='Blue',label = spacers)
plt.title('Counted vs. Predicted Fingers')

plt.xlim(0,80)
plt.ylim(0,80)
plt.xlabel('Count')
plt.ylabel('Prediction (Integration)')
plt.show()

## Plot predictions against counts for Rb_init ranges
# predicted vs counted values of n, divided into ranges of Rb_init

line_x = linspace(0,200,40)

Rb_div = linspace(min(Rb_init),max(Rb_init),5) # Divide set of initial R/b values into ranges
for j in range(len(Rb_div)-1): # plot each range of points one at a time, labelling ranges
    x, y, x_err, y_err, Rb_0 = [], [] ,[] ,[], []
    for i in range(len(n_count)): # produce arrays of n_int, n_count at each Rb range, plot
        if Rb_init[i]>=Rb_div[j] and Rb_init[i]<=Rb_div[j+1]:
            Rb_0 = Rb_0 + [Rb_init[i]]
            x = x + [n_count[i]]
            x_err = x_err + [n_count_err[i]]
            y = y + [n_int[i]]
    plt.errorbar(x, y, xerr=x_err, fmt='.', label=str(int(round(Rb_div[j])))+' < Initial R/b < '+str(int(round(Rb_div[j+1]))))

plt.plot(line_x,line_x,color='black') # plot a diagonal line to show deviation from prediction
plt.title('Counted vs. Predicted Fingers for 1.512 mm Plate Spacing')
plt.xlim(0,max([max(n_count),max(n_int)])+5) # set axes to include 0, max
plt.ylim(0,max([max(n_count),max(n_int)])+5)
plt.gca().set_aspect('equal', adjustable='box') # scale x and y axes equally
plt.draw()
plt.xlabel('Count')
plt.ylabel('Prediction (Integration)')
plt.legend(loc='lower right')
plt.show()

## If fewer points, plot Ca vs counted fingers labelled with initial R/b (see below)
for i in range(len(Ca_med)): # plot data points one at a time, labelling each with its initial R/b
    x = [Ca_med[i]]
    x_err = [Ca_err[i]]
    y = [n_count[i]]
    y_err = [n_count_err[i]]
    Rb_0 = Rb_init[i]
    plt.errorbar(x, y, xerr=x_err, yerr=y_err, fmt=' ', label='Initial R/b = '+str(round(Rb_0, decimals=2)))
    plt.title('Finger Count with ' + spacers + ' Spacers--Semilog')
    plt.xscale('log')
    plt.xlabel('Ca')
    plt.ylabel('n')
    plt.legend(loc='lower right')
    plt.show()

## Plot count with predictions for a specific R/b range
Rb_init_min = 16.0
Rb_init_max = 51.0

# n vs. Ca only for an Rb_init range stipulated above

x, y, x_err, y_err, y_int = [],[],[],[],[]
for i in range(len(Ca_med)):
    if Rb_init[i]>=Rb_init_min and Rb_init[i]<=Rb_init_max:
        x = x + [Ca_med[i]]
        x_err = x_err + [Ca_err[i]]
        y = y + [n_count[i]]
        y_err = y_err + [n_count_err[i]]
        y_int = y_int + [n_int[i]]
plt.errorbar(x, y, xerr=x_err, yerr=y_err, fmt=' ',label = 'Counted fingers')
plt.errorbar(x, y_int, xerr=x_err,fmt=' ',label = 'Prediction (integration)')

plt.title('Predicted (integration) and Counted Fingers with '+spacers+' Spacers and '+str(int(Rb_init_min))+'< R_0/b <' +str(int(Rb_init_max)))
plt.xscale('log')
plt.xlabel('Ca')
plt.ylabel('n')
plt.legend(loc='lower right')
plt.show()

## Same as above but for Reynolds number
Rb_init_min = 16.0
Rb_init_max = 51.0

# n vs. Re only for an Rb_init range stipulated above

x, y, x_err, y_err, y_int = [],[],[],[],[]
for i in range(len(Re_av)):
    if Rb_init[i]>=Rb_init_min and Rb_init[i]<=Rb_init_max:
        x = x + [Re_av[i]]
        x_err = x_err + [Re_std[i]]
        y = y + [n_count[i]]
        y_err = y_err + [n_count_err[i]]
        y_int = y_int + [n_int[i]]
plt.errorbar(x, y, xerr=x_err, yerr=y_err, fmt=' ',label = 'Counted fingers')
plt.errorbar(x, y_int, xerr=x_err,fmt=' ',label = 'Prediction (integration)')

plt.title('Predicted (integration) and Counted Fingers with '+spacers+' Spacers and '+str(int(Rb_init_min))+'< R_0/b <' +str(int(Rb_init_max)))
plt.xscale('log')
plt.xlabel('Re')
plt.ylabel('n')
plt.legend(loc='lower right')
plt.show()

## If many points, divide data by inital R/b range and plot Ca vs n with ranges labelled
# n vs Ca for all values of Rb_init, but colour coded into different Rb_init ranges

Rb_div = linspace(min(Rb_init),max(Rb_init),5) # Divide set of initial R/b values into ranges
for j in range(len(Rb_div)-1): # plot each range of points one at a time, labelling ranges
    x, y, x_err, y_err, Rb_0 = [], [] ,[] ,[], []
    for i in range(len(Ca_med)):
        if Rb_init[i]>=Rb_div[j] and Rb_init[i]<=Rb_div[j+1]:
            Rb_0 = Rb_0 + [Rb_init[i]]
            x = x + [Ca_med[i]]
            x_err = x_err + [Ca_err[i]]
            y = y + [n_count[i]]
            y_err = y_err + [n_count_err[i]]
    plt.errorbar(x, y, xerr=x_err, yerr=y_err, fmt=' ', label=str(int(round(Rb_div[j])))+' < Initial R/b < '+str(int(round(Rb_div[j+1]))))

    
plt.title('Finger Count with ' +spacers+' Spacers--Semilog')
plt.xscale('log')
plt.xlabel('Ca')
plt.ylabel('n')
plt.legend(loc='lower right')
plt.show()

## Plot Re vs counted fingers, labelled by initial R/b range divided linearly or ogarithmically
# n vs Re for all values of Rb_init, but colour coded into different Rb_init ranges, which can be divided up into equal ranges either in linspace or logspace

log = False # toggle whether to divide initial R/b ranges linearly or logarithmically
if log == True:
    Rb_div = logspace(log10(min(Rb_init)),log10(max(Rb_init)),5) # Divide set of initial R/b values into ranges
else:
    Rb_div = linspace(min(Rb_init),max(Rb_init),5)
    
for j in range(len(Rb_div)-1): # plot each range of points one at a time, labelling ranges
    x, y, x_err, y_err, Rb_0 = [], [] ,[] ,[], []
    for i in range(len(Re_av)):
        if Rb_init[i]>=Rb_div[j] and Rb_init[i]<=Rb_div[j+1]:
            Rb_0 = Rb_0 + [Rb_init[i]]
            x = x + [Re_av[i]]
            x_err = x_err + [Re_std[i]]
            y = y + [n_count[i]]
            y_err = y_err + [n_count_err[i]]
    plt.errorbar(x, y, xerr=x_err, yerr=y_err, fmt=' ', label=str(int(round(Rb_div[j])))+' < Initial R/b < '+str(int(round(Rb_div[j+1]))))

plt.title('Finger Count vs Reynolds Number with ' +spacers+' Spacers--Semilog')
plt.xscale('log')
plt.xlabel('Re')
plt.ylabel('n')
plt.legend(loc='upper left')
plt.show()


## Master coverage plot
# plot points in Ca-Rb_init space, labelled with finger count

plt.errorbar(Rb_init, Ca_med, fmt = '.', yerr=Ca_err, color='blue')
for x,y,z in zip(Rb_init, Ca_med, n_count):
    plt.text(x, y, str(z), color='black', fontsize=12)
plt.yscale('log')
plt.title('Capillary Number vs. Initial R/b, Labelled with Finger Count')
plt.xlabel('Initial R/b')
plt.ylabel('Capillary number')
plt.show()

## Reynolds coverage plot
# plot points in Re-Rb_init space, labelled with finger count

plt.errorbar(Rb_init, Re_av, fmt = '.', yerr=Re_std, color='blue')
for x,y,z in zip(Rb_init, Re_av, n_count):
    plt.text(x, y, str(z), color='black', fontsize=12)
plt.yscale('log')
plt.title('Reynolds Number vs. Initial R/b, Labelled with Finger Count')
plt.xlabel('Initial R/b')
plt.ylabel('Reynolds Number')
plt.show()

## Averaged coverage plot
# Average over nearby points in Ca-Rb_init space, and plot points, labelled with average finger count and the number of runs contributing to that averaged value. 

# Set delimiters to split up Rb_init, Ca_med data into grid
Rb_delims = [0.0,40.0,60.0,80.0,100.0,max(Rb_init)+1]
Ca_delims = [0.0,10.0**-1.5,10.0**-1.0,10.0**-0.5,1.0, max(Ca_med)+1]

# Create empty lists corresponding to the delimitation above
Rb_init_grid = [[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]]]
Ca_med_grid = [[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]]]
n_count_grid = [[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]],[[],[],[],[],[]]]

# Run through the data and group into grid cells
for i in range(len(Rb_init)):
    for j in range(len(Rb_delims)-1):
        if Rb_init[i]>Rb_delims[j] and Rb_init[i]<Rb_delims[j+1]:
            for k in range(len(Ca_delims)-1):
                if Ca_med[i]>Ca_delims[k] and Ca_med[i]<Ca_delims[k+1]:
                    Rb_init_grid[j][k] = Rb_init_grid[j][k] + [Rb_init[i]]
                    Ca_med_grid[j][k] = Ca_med_grid[j][k] + [Ca_med[i]]
                    n_count_grid[j][k] = n_count_grid[j][k] + [n_count[i]]

# Average over lists in the grid arrays to create 2-d lists of averages
Rb_init_averages = [[],[],[],[],[]]
Ca_med_averages = [[],[],[],[],[]]
n_count_averages = [[],[],[],[],[]]
n_count_stderr = [[],[],[],[],[]]
sample_number = [[],[],[],[],[]]

for j in range(len(Rb_delims)-1):
    for k in range(len(Ca_delims)-1):
        Rb_init_averages[j] = Rb_init_averages[j] + [average(Rb_init_grid[j][k])]
        Ca_med_averages[j] = Ca_med_averages[j] + [average(Ca_med_grid[j][k])]
        n_count_averages[j] = n_count_averages[j] + [average(n_count_grid[j][k])]
        n_count_stderr[j] = n_count_stderr[j] + [std(n_count_grid[j][k])/sqrt(len(n_count_grid[j][k]))]
        sample_number[j] = sample_number[j] + [len(n_count_grid[j][k])]
        
# Flatten the lists we created above for plotting        
Rb_init_averages = list(chain.from_iterable(Rb_init_averages))
Ca_med_averages = list(chain.from_iterable(Ca_med_averages))
n_count_averages = list(chain.from_iterable(n_count_averages))
n_count_stderr = list(chain.from_iterable(n_count_stderr))
sample_number = list(chain.from_iterable(sample_number))

# Create the plot with points representing averaged grid cells
plt.plot(Rb_init_averages, Ca_med_averages,'.')
for x,y,z1,z2,z3 in zip(Rb_init_averages, Ca_med_averages, n_count_averages, n_count_stderr, sample_number):
    plt.text(x, y, '(' + str(z3) + '), ' + str(round(z1)) + '+/-' + str(round(z2,1)), color='black', fontsize=12)
plt.yscale('log')
plt.title('Capillary Number vs. Initial R/b, Labelled with Finger Count and Sample Size, Averaged Over 2-d Regions')
plt.xlabel('Initial R/b')
plt.ylabel('Capillary number')
plt.show()


## Plot predicted and counted modified wavenumber (Maxworthy) on semilog plot
# This plot did not really work because we could not really define a modified wave number

A_count = (n_count/sqrt(Ca_med))*(median(bR))
A_count_err = (n_count_err/sqrt(Ca_med))*(median(bR))
A_init = (n_init/sqrt(Ca_init))*(bR[0])
A_int = (n_int/sqrt(Ca_med))*(median(bR))

plt.errorbar(Ca_med, A_count, xerr=Ca_err, yerr=A_count_err, fmt = ' ', color='blue',label='Counted fingers')
plt.plot(Ca_init, A_init, 'o', color='green',label='Prediction (initial)')
plt.errorbar(Ca_med, A_int, xerr=Ca_err, fmt = ' ', color='red',label='Prediction (integration)')
plt.title('Modified Wavenumber and Predictions with ' + spacers + ' Spacers--Semilog')
plt.xscale('log')
plt.xlabel('Ca')
plt.ylabel('$A_{max} = nbR^{-1}Ca^{-1/2}$')
plt.legend(loc = 'lower right')
plt.show()