'''
This code takes binarized FVF images from runs of the FVF experiment at a variety of different initial bubble sizes 
to plot velocity against radius. We want an empirical record of how interface velocity depends on bubble radius at 
a variety of different pressures, plate spacings, etc., so we can consistently achieve a variety of different values 
of capillary number, and so we can make more sophisticated  estimates for the mode of the interface instability with 
the maximum growth rate by integrating over a range of velocities (capillary numbers) throughout the linear regime.

Manually process the data from a variety of different text files generated by FVF_getradius.py. Compile their velocities 
and radii into two arrays and plot.

Written by: Matthew Gerry
October, 2018
'''

import sys
sys.path.append('FVF_code') # add to sys.path wherever FVF-related scripts are saved
from numpy import loadtxt, polyfit, polyder, polyval, concatenate, round
import matplotlib.pyplot as plt
from viscosity_vs_T import *

## Set up arrays to hold radius and velocity data from a variety of runs at the same pressure, plate spacing

R = [] # radii throughout the circular regime
v = [] # velocities throughout the circular regime
Ca = [] # capillary number, calculated from velocity
Re = [] # Reynolds number, calculated from velocity and plate spacing

# spacer dimensions, in m
spacer_dim = {'A': 7.91e-4, 'B': 1.051e-3, 'C': 1.303e-3, 'D': 1.512e-3, 'E': 2.334e-3, 'F': 3.206e-3, 'G': 5.2e-4, 'H': 3.2e-4}

spacers = str('G') # label of spacers used
b = spacer_dim[spacers] # the plate spacing in meters

p = 20.0 # the pressure in psi
needle = 'open' # needle position, based on markings (10 is about a full turn)

temperature = 22.0  # the temperature of the oil in degrees C
rho = 8.6e2 # the density of the oil in kg/m^3

gamma = 0.033       					# the surface tension in N/m.  See Table 1.
Ca_ratio = mu_vs_T(temperature)/gamma	# = mu/gamma in (s/m).  The conversion factor of velocity in (m/s) to capillary number Ca. 
Re_ratio = b*rho/mu_vs_T(temperature)

## Set up data of a specific run for R, v analysis--repeat for each run in the set
# Import data; take note of decimation, etc.
path_to_txt_name = 'FVF_data/'   #  a string specifying the path to the txt file of the mean radius data.
input_txt_name = '/'  # the name of the txt file with the mean radius data.

# Specify details for unit conversions, specific to the data file imported above
decimate =  1   # this has to match the value used in process_FVF_images.py
frame_rate = 120.0  # the number of frames per second the camera was originally set to
sec_per_frame = decimate*(1.0/frame_rate)  #  Note how decimation increases the time between frames
mm_per_pixel = 20.0/70.0  # Count xx mm for yy pixels from your calibration image showing the plastic ruler            
meter_per_pixel = mm_per_pixel/1000.0  # better units

# Construct initial radius, velocity arrays
# Fit the radius data to a ploynomial; differentiate for velocity data (see code linear_theory_FVF.py)
order_of_polyfit = 4  # The order of the polynomial fit. Make this big enough to make a tight fit, but not more than that.

# While we are only considering initial radii and velocities at the moment, use data from the entire linear regime for a valid polynomial fit from which velocity values are derived

# retrieve mean radius data from the txt file that was created by mean_radius_FVF.py
(frame_num, rad_pixels, stdevn, rmax, rmin) = loadtxt(path_to_txt_name + input_txt_name)

frame_start = 0     # this will be zero unless you started from a static circular bubble
frame_stop = 12     # inclusive  (add +1 below), the frame at which the linear regime ends in the given run

# Truncate the data to just the linear regime and do units conversion from pixels/frame to m/s
time = sec_per_frame * frame_num[ frame_start:frame_stop+1 ]  		# array of times at which the radius data were collected, now in seconds
radii = meter_per_pixel * rad_pixels[ frame_start:frame_stop+1 ]	# array of radii, now in meters

# carry out the polynomial fit. This is just a phenomenological fit to allow us to differentiate the data nicely
R_fit_coef = polyfit(time, radii, order_of_polyfit)  

# fit the coefficients for the derivative of the polynomial fit --> a polynomial for v = dR/dt
v_fit_coef = polyder(R_fit_coef)

initial_radius = polyval(R_fit_coef, time)[0] #value should be similar to radii[0]
initial_velocity = polyval(v_fit_coef, time)[1]

R = R + [polyval(R_fit_coef, time)]
v = v + [polyval(v_fit_coef, time)]

#compute capillary number from velocity data
Ca = Ca + [Ca_ratio*polyval(v_fit_coef, time)]
Re = Re + [Re_ratio*polyval(v_fit_coef, time)]

## Plot v_i vs. R_0 data
# Once data from a variety of runs has been processed in the cells above, plot initial v against R

for i in range(len(R)):
 plt.plot(R[i][0],v[i][0], '.', color='blue')

plt.title('Initial Velocities vs Initial Radii at p=' + str(p) + ', needle =' + str(needle) + ', and b=' + str(b))
plt.xlabel('Initial radius (m)')
plt.ylabel('Initial interface velocity (m/s)')
plt.show()

## Plot v vs R data
# Plot the full velocity and radius arrays

for i in range(len(R)):
    plt.plot(R[i],v[i],'.',label = 'R_0 = '+str(round(R[i][0],4)) +' m')

plt.title('Velocities vs Radii at p=' + str(p) + ' psi, needle =' + str(needle) + ', and b=' + 'b' + ' m, '+str(order_of_polyfit) +' dof polyfit')
plt.xlabel('Radius (m)')
plt.ylabel('Interface velocity (m/s)')
plt.legend(loc='upper right')

#plt.savefig(path_to_txt_name+input_txt_name+'_v_vs_R.pdf')
plt.show()

## Plot Ca vs R data
# The same plot as above, but with velocity converted to Ca

for i in range(len(R)):
    plt.plot(R[i],Ca[i],'.',label = 'Initial R = '+str(round(R[i][0],4)) +' m')

#plt.title('Capillary Number vs Radius at p=' + str(p) + ' psi, needle =' + str(needle) + ', and b=' + str(b) + ' m, '+str(order_of_polyfit) +' dof polyfit')

plt.title('Capillary Number vs Radius at 2.0 psi, Needle Valve < 1/2 turn open, 0.52(2) mm plate spacing (4 dof polynomial fit)')
plt.xlabel('Radius (m)')
plt.ylabel('Capillary Number')
plt.legend(loc='upper right')

#plt.savefig(path_to_txt_name+input_txt_name+'_Ca_vs_R.pdf')
plt.show()

## Plot Re vs R data
# The same plot as above, but with velocity converted to Re. This will differ with the same speeds at different plate spacing.

for i in range(len(R)):
    plt.plot(R[i],Re[i],'.',label = 'Initial R = '+str(round(R[i][0],4)) +' m')

#plt.title('Reynolds Number vs Radius at p=' + str(p) + ' psi, needle =' + str(needle) + ', and b=' + str(b) + ' m, '+str(order_of_polyfit) +' dof polyfit')
plt.title('Reynolds Number vs Radius at 2.0 psi, Needle Valve < 1/2 turn open, 0.52(2) mm plate spacing (4 dof polynomial fit)')
plt.xlabel('Radius (m)')
plt.ylabel('Reynolds Number')
plt.legend(loc='upper right')

#plt.savefig(path_to_txt_name+input_txt_name+'_Re_vs_R.pdf')
plt.show()